function [coord in]=hcfft_coordsg(d, n, in)
% Computes the coordinates of a sparse grid of dimension d and level n
% in the order of the indexing function.
%
% [coord in]=hcfft_coordsg(d,n,in)
%
% d     = dimension
% n     = level of sparse grid
% in    = (struct with some initial matrices) is an optional argument
%
% Example:
%
%   s=hcfft_coordsg(4,3)    % gives Nx4 matrix, rows are sg points
%                           % of dimension 4 and level 3
%
% See also: hcfft_vec2sg, hcfft_sg2vec
%
% Copyright 2009 (c) Michael Doehler, Lutz Kaemmerer, Stefan Kunis, Daniel Potts
%
% $Id: hcfft_coordsg.m 1102 2010-07-08 14:29:25Z kaemmerer $

    if(nargin==2)
        in=hcfft_initialize(d,n);
        %disp('initialization in hcfft_coordsg');
    end
    l=zeros(1,d);
    global HCFFT_coord;
    HCFFT_coord=zeros(in.s(d, n+1),d);
    for k=0:n
        numl(l, 1, d, k, n, in);
    end
    coord = HCFFT_coord;
    clear HCFFT_coord;
end

function numl(l, d1, d, k, n, in)
    m=0;
    if(d1>1)
       m=sum(l(1:d1-1));
    end
    for i=0:k-m
        l(d1)=i;
        if(d1<d-1)
            numl(l, d1+1, d, k, n, in);
        else
            l(d)=k-m-i;
            q=zeros(1,d);
            numq(q, 1, d, l, n, k, in);
        end 
    end
end

function numq(q, d1, d, l, n, k, in)
    global HCFFT_coord;
    for i=1:ceil(2^(l(d1)-1))
        if(l(d1)==0)
            q(d1)=0;
        else
            q(d1)=2*i-1;
        end
        if(d1<d)
            numq(q, d1+1, d, l, n, k, in);
        else
            ind=hcfft_index(l, q, d, k, in);
            HCFFT_coord(ind,:)=q./2.^l;
        end
    end            
end
