function f=lhcfft(H,z,M,f_hat,tflag)
%
% f=lhcfft(H,z,M,f_hat,tflag)
%
%   H:      hyperbolic cross index matrix
%   z:      generating integer vector
%   M:      lattice size
%   fhat:   Fourier coefficients f_hat(j)=f_hat_H(j,:) with tflag='notransp'
%           Function values f_hat(j)=f(mod(j*z/M,1)) with tflag='transp'
%   tflag:  'notransp' -> Evaluation of 
%
%               f(j+1)=sum_{k\in H_n^d}f_hat(k)*e^(-2*pi*i*j*k'z/M), j=0:M-1
%
%           'transp' -> Evaluation of transposed
%               f(l)=sum_{j=0}^M f_hat(j)*e^(2*pi*i*j*k(l)'z/M),
%                                           k(l)=H(l,:), l=1:size(H,1)
%
% See also: hcfft, hcdft, nhcdft, nhcfft, lhcfft_init
%
% Copyright 2010 (c) Michael Doehler, Lutz Kaemmerer, Stefan Kunis, Daniel Potts
%
% $Id: lhcfft.m 1102 2010-07-08 14:29:25Z kaemmerer $               

    switch(tflag)
        case 'notransp'
            f=lhcfft_notransp(H,z,M,f_hat);
        case 'transp'
            f=lhcfft_transp(H,z,M,f_hat);
    end

end

function f=lhcfft_notransp(H,z,M,f_hat)

    k=mod(H*z',M)+1;
    fhat1=accumarray(k,f_hat,[M,1],@sum);
%    for j=1:length(k)
%        fhat1(k(j))=fhat1(k(j))+f_hat(j);
%    end; toc
    
    f=M*ifft(fhat1);


end

function fhat=lhcfft_transp(H,z,M,f)

    ghat=M*ifft(conj(f));
    k=mod(H*z',M)+1;
    fhat=zeros(1,length(k));
    for j=1:length(k)
        fhat(j)=ghat(k(j));
    end
    
end