function [H,z,M]=lhcfft_init(d,n,type)
%
%
% [H,z,M]=lhcfft_init(d,n,type)
%
%   d:      dimension of considered hyperbolic cross
%   n:      refinement of considered hyperbolic cross
%   type:   one of the search strategies
%           ('global','random','korobov','randkorobov', 'fixedkorobov') 
%           or 'min'  
%
%   returns the index matrix H of hyperbolic cross of dimension d and 
%   refinement n and
%   the generating integer vector z and lattice size M of a nonaliasing
%   lattice for the hyperbolic cross found by the given searchtype
%   if there is no such vector, error occurs
%
% See also: lhcfft, lhcfft_search
%
% Copyright 2010 (c) Michael Doehler, Lutz Kaemmerer, Stefan Kunis, Daniel Potts
%
% $Id: lhcfft_init.m 1102 2010-07-08 14:29:25Z kaemmerer $

    if(exist('lhcfft_init_data_modified','file'))
        load('lhcfft_init_data_modified');
    else
        load('lhcfft_init_data');
    end
    
    % maybe if exists one can search through lhcfft_init_data_modified
    % look at lhcfft_search for details
    
    switch(type)
        case 'min'
            test={'fixedkorobov','korobov','randkorobov','global','random'};
            z=zeros(5,d);
            M=zeros(1,5);
            for j=1:5
                wsb=warning('query','lhcfft_init:NoSuchLattice');
                warning('OFF','lhcfft_init:NoSuchLattice');
                [H1,z(j,:),M(j)]=lhcfft_init(d,n,test{j});
                if(size(H1,1)>1)
                    H=H1;
                end
                warning(wsb);
            end
            if(length(M(M>0))==0)
                warning('lhcfft_init:NoSuchLattice','There is no nonaliasing lattice available so far.');
                M=0; z=zeros(1,d); H=z;
            else
                M(M==0)=max(M)+1;
                [M,I]=min(M);
                z=z(I,:);
            end
            
            
            
        otherwise
            
            if(isfield(Lattices,sprintf('d%d',d)) && isfield(Lattices.(sprintf('d%d',d)),type))
                lat=Lattices.(sprintf('d%d',d)).(type);
                if(length(lat.M)>=n && lat.M(n)~=0)
                    H=hcfft_coordhc(d,n);
                    M=lat.M(n);
                    switch(type)
                        case 'fixedkorobov'
                            if(n==1)
                                z=1:d;
                            else
                                z=zeros(1,d);
                                z(1)=1;
                                for j=2:d
                                    z(j)=mod(z(j-1)*3*2^(n-2),M);
                                end
                            end
                        otherwise
                            z=mod(lat.z(n,:),M);
                    end
                else
                    warning('lhcfft_init:NoSuchLattice','There is no nonaliasing lattice of type ''%s'' available for this problem size so far.',type);
                    M=0;
                    z=zeros(1,d);
                    H=hcfft_coordhc(d,n);
                end
            else
                warning('lhcfft_init:NoSuchLattice','There is no nonaliasing lattice of type ''%s'' available for this problem size so far.',type);
                M=0;
                z=zeros(1,d);
                H=z;
            end
    end
end