% Script nhcfft_error.m; Example for the paper
% Nonequispaced hyperbolic cross fast Fourier transform
%
% Relative error of NHCFFT with respect to spline order, produces Figure 5.2
%
% Copyright 2009 (c) Michael Doehler, Lutz Kaemmerer, Stefan Kunis, Daniel Potts
%
% $Id: nhcfft_error.m 1102 2010-07-08 14:29:25Z kaemmerer $

dd=[2,3];
L=[3,2];
nn={[6,10,14],[6,10]};

mm=2:2:14;

for l=1:2;
  d=dd(l);
  alpha=d;

  % measured error
  E=zeros(L,length(mm));
  % Remark 4.2
  E1=zeros(L,length(mm));
  % Theorem 4.1
  E2=zeros(L,length(mm));

  for r=1:L
    n=nn{l}(r)
    r=n+alpha;

    in = hcfft_initialize(d, n);
    N=in.s(d, n+1);
    f_hat=randn(N,1)+i*randn(N,1);

    x=rand(N,d);
      
    f1=nhcdft(f_hat,x,'notransp',in);

    in = hcfft_initialize(d, n+alpha);

    for s=1:length(mm)
      m=mm(s);
      
      f2=nhcfft(f_hat,x,m,alpha,'notransp',in);

      E(r,s)=norm(f1-f2,inf)/norm(f_hat,1);

      disp(sprintf('d=%d,\talpha=%d\tn=%d\tr=%d\tm=%d\te=%1.2e\n',d,alpha,n,r,m,E(r,s)));

      % Remark 4.2, F_m <= 4/pi
      if(d==2)
        E1(r,s)=(r/2^((r-1)*m)+(r+3)/2^(r*m))*(4/pi)^2*2^(n*m);
      end;

      if(d==3)
        E1(r,s)=0.5*((r-1)*r/2^((r-2)*m)+2*r*(r+4)/2^((r-1)*m)+(r+2)*(r+6)/2^(r*m))*(4/pi)^3*2^(n*m);
      end;

      % Theorem 4.1, F_m <= 4/pi
      E2(r,s)=(2*r+2)^(d-1)*(4/pi)^d*2.^((n-(r-d+1))*m);
    end;
  end;

  if(d==2)
    h=semilogy(mm,E(1,:),'-+k',...
	       mm,E1(1,:),'-sk',...
	       mm,E2(1,:),'-dk',...
	       mm,E(2,:),'-.+k',...
	       mm,E1(2,:),'-.sk',...
	       mm,E2(2,:),'-.dk',...
	       mm,E(3,:),':+k',...
	       mm,E1(3,:),':sk',...
	       mm,E2(3,:),':dk');
    set(h,'LineWidth',2); set(h,'MarkerSize',8); set(gca,'FontSize',15);

    % copy plot for legend
    hold on
    k=semilogy(mm,E(1,:),'+k',...
	       mm,E1(1,:),'sk',...
	       mm,E2(1,:),'dk',...
	       mm,E(1,:),'-k',...
	       mm,E(2,:),'-.k',...
	       mm,E(3,:),':k');
    hold off
    set(k,'LineWidth',2); set(h,'MarkerSize',8); set(gca,'FontSize',15);
    legend(k,'measured', 'Remark 4.2','Theorem 4.1','n=6','n=10','n=14');

    xlabel('Spline order m');
    ylabel('Relative error');
    ylim([5e-9 1e2]);

    print nhcfft_error_r2_2d.png -dpng
  end;

  if(d==3)
    h=semilogy(mm,E(1,:),'-+k',...
	       mm,E1(1,:),'-sk',...
	       mm,E2(1,:),'-dk',...
	       mm,E(2,:),'-.+k',...
	       mm,E1(2,:),'-.sk',...
	       mm,E2(2,:),'-.dk')
    set(h,'LineWidth',2); set(h,'MarkerSize',8); set(gca,'FontSize',15);

    % copy plot for legend
    hold on
    k=semilogy(mm,E(1,:),'+k',...
	       mm,E1(1,:),'sk',...
	       mm,E2(1,:),'dk',...
	       mm,E(1,:),'-k',...
	       mm,E(2,:),'-.k');
    hold off
    set(k,'LineWidth',2); set(h,'MarkerSize',8); set(gca,'FontSize',15);
    legend(k,'measured', 'Remark 4.2','Theorem 4.1','n=6','n=10');

    xlabel('Spline order m');
    ylabel('Relative error');
    ylim([5e-9 1e2]);

    print nhcfft_error_r2_3d.png -dpng
  end;
end;
