% Script nhcfft_times.m; Example for the paper
% Nonequispaced hyperbolic cross fast Fourier transform
%
% Computational times of the NHCDFT, the NHCFFT, and its three steps with
% respect to the refinement n, produces Figure 5.3
%
% Copyright 2009 (c) Michael Doehler, Lutz Kaemmerer, Stefan Kunis, Daniel Potts
%
% $Id: nhcfft_times.m 1102 2010-07-08 14:29:25Z kaemmerer $

dd=[2,3];
nn=1:13;
m=4;
T=zeros(13,5);

print_names={'test_nhcfft_times_2d_4','test_nhcfft_times_3d_4'};

for l=1:2;
  d=dd(l);
  alpha=d;

  disp(sprintf('\nd=%d Computational time',d));
  disp(sprintf('n\tHCFFT\t\tInterpolation\tEvaluation\tNHCFFT total\tNHCDFT\t\trel. error'));


  for n=nn    
    r=n+alpha;

    in0 = hcfft_initialize(d, n);
    N0=in0.s(d, n+1);
    f_hat0=randn(N0,1)+i*randn(N0,1);

    x=rand(N0,d);

    in = hcfft_initialize(d, n+alpha);
    N = in.s(d, n+alpha+1);
    f_hat=zeros(N,1);
    f_hat(1:N0)=f_hat0;
    
    t=cputime;
    g=hcfft(f_hat,'notransp','noinv',in);
    T(n,1)=cputime-t;

    t=cputime;
    a=nhcfft_bool_int(g,m,'notransp',in);
    T(n,2)=cputime-t;

    t=cputime;
    s=nhcfft_bool_val(a,m,x,'notransp');
    T(n,3)=cputime-t;

    t=cputime;
    s2=nhcfft(f_hat0,x,m,alpha,'notransp',in);
    T(n,4)=cputime-t;

    t=cputime;
    f=nhcdft(f_hat0,x,'notransp',in0);
    T(n,5)=cputime-t;

    e=norm(s2-f,inf)/norm(f_hat0,1);

    disp(sprintf('%d\t%1.2e\t%1.2e\t%1.2e\t%1.2e\t%1.2e\t%1.2e',n,T(n,:),e));
  end;

  h=semilogy(nn,T(:,1),':+k',...
             nn,T(:,2),':^k',...
             nn,T(:,3),':vk',...
             nn,T(:,4),'-+k',...
             nn,T(:,5),'--sk');

  set(h,'LineWidth',2); set(h,'MarkerSize',8); set(gca,'FontSize',15);
  xlabel('Refinement n');
  ylabel('Computational time in seconds');
  legend('HCFFT', 'Interpolation', 'Evaluation', 'NHCFFT total', 'NHCDFT','Location','NorthWest');
  axis([1,13,10^-2,10^4]);

  print('-dpng',print_names{l});
end;    
